@@ -1,17 +1,20 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2014 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2015 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from many
-   others, a complete list can be found in the file AUTHORS.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
    You may use this file and others of this release according to the
    license defined in the LICENSE file, which includes the Affero General
    Public License, v3.0 ("AGPLv3") and some additional permissions and
    terms pursuant to its AGPLv3 Section 7.
 
-   Bacula® is a registered trademark of Kern Sibbald.
+   This notice must be preserved when any source code is 
+   conveyed and/or propagated.
+
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
  *   Main configuration file parser for Bacula File Daemon (Client)
@@ -32,7 +35,7 @@
  *      definitions as well as any specific store routines
  *      for the resource records.
  *
- *     Written by Kern Sibbald, September MM
+ *     Kern Sibbald, September MM
  *
  */
 
@@ -77,64 +80,65 @@ static void store_digest_type(LEX *lc, RES_ITEM *item, int index, int pass);
 
 /* Client or File daemon "Global" resources */
 static RES_ITEM cli_items[] = {
-   {"name",        store_name,  ITEM(res_client.hdr.name), 0, ITEM_REQUIRED, 0},
-   {"description", store_str,   ITEM(res_client.hdr.desc), 0, 0, 0},
-   {"fdport",      store_addresses_port,    ITEM(res_client.FDaddrs),  0, ITEM_DEFAULT, 9102},
-   {"fdaddress",   store_addresses_address, ITEM(res_client.FDaddrs),  0, ITEM_DEFAULT, 9102},
-   {"fdaddresses", store_addresses,         ITEM(res_client.FDaddrs),  0, ITEM_DEFAULT, 9102},
-   {"fdsourceaddress", store_addresses_address, ITEM(res_client.FDsrc_addr),  0, ITEM_DEFAULT, 0},
+   {"Name",        store_name,  ITEM(res_client.hdr.name), 0, ITEM_REQUIRED, 0},
+   {"Description", store_str,   ITEM(res_client.hdr.desc), 0, 0, 0},
+   {"FdPort",      store_addresses_port,    ITEM(res_client.FDaddrs),  0, ITEM_DEFAULT, 9102},
+   {"FdAddress",   store_addresses_address, ITEM(res_client.FDaddrs),  0, ITEM_DEFAULT, 9102},
+   {"FdAddresses", store_addresses,         ITEM(res_client.FDaddrs),  0, ITEM_DEFAULT, 9102},
+   {"FdSourceAddress", store_addresses_address, ITEM(res_client.FDsrc_addr),  0, ITEM_DEFAULT, 0},
 
-   {"workingdirectory",  store_dir, ITEM(res_client.working_directory), 0, ITEM_REQUIRED, 0},
-   {"piddirectory",  store_dir,     ITEM(res_client.pid_directory),     0, ITEM_REQUIRED, 0},
-   {"subsysdirectory",  store_dir,  ITEM(res_client.subsys_directory),  0, 0, 0},
-   {"plugindirectory",  store_dir,  ITEM(res_client.plugin_directory),  0, 0, 0},
-   {"scriptsdirectory", store_dir,  ITEM(res_client.scripts_directory),  0, 0, 0},
-   {"maximumconcurrentjobs", store_pint32,  ITEM(res_client.MaxConcurrentJobs), 0, ITEM_DEFAULT, 20},
-   {"messages",      store_res, ITEM(res_client.messages), R_MSGS, 0, 0},
-   {"sdconnecttimeout", store_time,ITEM(res_client.SDConnectTimeout), 0, ITEM_DEFAULT, 60 * 30},
-   {"heartbeatinterval", store_time, ITEM(res_client.heartbeat_interval), 0, ITEM_DEFAULT, 0},
-   {"maximumnetworkbuffersize", store_pint32, ITEM(res_client.max_network_buffer_size), 0, 0, 0},
+   {"WorkingDirectory",  store_dir, ITEM(res_client.working_directory), 0, ITEM_REQUIRED, 0},
+   {"PidDirectory",  store_dir,     ITEM(res_client.pid_directory),     0, ITEM_REQUIRED, 0},
+   {"SubsysDirectory",  store_dir,  ITEM(res_client.subsys_directory),  0, 0, 0},
+   {"PluginDirectory",  store_dir,  ITEM(res_client.plugin_directory),  0, 0, 0},
+   {"SnapshotCommand",  store_str,  ITEM(res_client.snapshot_command), 0, 0, 0},
+   {"ScriptsDirectory", store_dir,  ITEM(res_client.scripts_directory),  0, 0, 0},
+   {"MaximumConcurrentJobs", store_pint32,  ITEM(res_client.MaxConcurrentJobs), 0, ITEM_DEFAULT, 20},
+   {"Messages",      store_res, ITEM(res_client.messages), R_MSGS, 0, 0},
+   {"SdConnectTimeout", store_time,ITEM(res_client.SDConnectTimeout), 0, ITEM_DEFAULT, 60 * 30},
+   {"HeartbeatInterval", store_time, ITEM(res_client.heartbeat_interval), 0, ITEM_DEFAULT, 5 * 60},
+   {"MaximumNetworkBufferSize", store_pint32, ITEM(res_client.max_network_buffer_size), 0, 0, 0},
 #ifdef DATA_ENCRYPTION
-   {"pkisignatures",         store_bool,    ITEM(res_client.pki_sign), 0, ITEM_DEFAULT, 0},
-   {"pkiencryption",         store_bool,    ITEM(res_client.pki_encrypt), 0, ITEM_DEFAULT, 0},
-   {"pkikeypair",            store_dir,       ITEM(res_client.pki_keypair_file), 0, 0, 0},
-   {"pkisigner",             store_alist_str, ITEM(res_client.pki_signing_key_files), 0, 0, 0},
-   {"pkimasterkey",          store_alist_str, ITEM(res_client.pki_master_key_files), 0, 0, 0},
-   {"pkicipher",             store_cipher_type, ITEM(res_client.pki_cipher), 0, 0, 0},
-   {"pkipigest",             store_digest_type, ITEM(res_client.pki_digest), 0, 0, 0},
+   {"PkiSignatures",         store_bool,    ITEM(res_client.pki_sign), 0, ITEM_DEFAULT, 0},
+   {"PkiEncryption",         store_bool,    ITEM(res_client.pki_encrypt), 0, ITEM_DEFAULT, 0},
+   {"PkiKeyPair",            store_dir,         ITEM(res_client.pki_keypair_file), 0, 0, 0},
+   {"PkiSigner",             store_alist_str,   ITEM(res_client.pki_signing_key_files), 0, 0, 0},
+   {"PkiMasterKey",          store_alist_str,   ITEM(res_client.pki_master_key_files), 0, 0, 0},
+   {"PkiCipher",             store_cipher_type, ITEM(res_client.pki_cipher), 0, 0, 0},
+   {"PkiDigest",             store_digest_type, ITEM(res_client.pki_digest), 0, 0, 0},
 #endif
-   {"tlsauthenticate",       store_bool,    ITEM(res_client.tls_authenticate),  0, 0, 0},
-   {"tlsenable",             store_bool,    ITEM(res_client.tls_enable),  0, 0, 0},
-   {"tlsrequire",            store_bool,    ITEM(res_client.tls_require), 0, 0, 0},
-   {"tlscacertificatefile",  store_dir,       ITEM(res_client.tls_ca_certfile), 0, 0, 0},
-   {"tlscacertificatedir",   store_dir,       ITEM(res_client.tls_ca_certdir), 0, 0, 0},
-   {"tlscertificate",        store_dir,       ITEM(res_client.tls_certfile), 0, 0, 0},
-   {"tlskey",                store_dir,       ITEM(res_client.tls_keyfile), 0, 0, 0},
-   {"verid",                 store_str,       ITEM(res_client.verid), 0, 0, 0},
-   {"maximumbandwidthperjob",store_speed,   ITEM(res_client.max_bandwidth_per_job), 0, 0, 0},
-   {"disablecommand",        store_alist_str, ITEM(res_client.disable_cmds), 0, 0, 0},
+   {"TlsAuthenticate",       store_bool,    ITEM(res_client.tls_authenticate),  0, 0, 0},
+   {"TlsEnable",             store_bool,    ITEM(res_client.tls_enable),  0, 0, 0},
+   {"TlsRequire",            store_bool,    ITEM(res_client.tls_require), 0, 0, 0},
+   {"TlsCaCertificateFile",  store_dir,     ITEM(res_client.tls_ca_certfile), 0, 0, 0},
+   {"TlsCaCertificateDir",   store_dir,     ITEM(res_client.tls_ca_certdir), 0, 0, 0},
+   {"TlsCertificate",        store_dir,     ITEM(res_client.tls_certfile), 0, 0, 0},
+   {"TlsKey",                store_dir,     ITEM(res_client.tls_keyfile), 0, 0, 0},
+   {"VerId",                 store_str,     ITEM(res_client.verid), 0, 0, 0},
+   {"MaximumBandwidthPerJob",store_speed,   ITEM(res_client.max_bandwidth_per_job), 0, 0, 0},
+   {"DisableCommand",        store_alist_str, ITEM(res_client.disable_cmds), 0, 0, 0},
    {NULL, NULL, {0}, 0, 0, 0}
 };
 
 /* Directors that can use our services */
 static RES_ITEM dir_items[] = {
-   {"name",        store_name,     ITEM(res_dir.hdr.name),  0, ITEM_REQUIRED, 0},
-   {"description", store_str,      ITEM(res_dir.hdr.desc),  0, 0, 0},
-   {"password",    store_password, ITEM(res_dir.password),  0, ITEM_REQUIRED, 0},
-   {"address",     store_str,      ITEM(res_dir.address),   0, 0, 0},
-   {"monitor",     store_bool,   ITEM(res_dir.monitor),   0, ITEM_DEFAULT, 0},
-   {"tlsauthenticate",      store_bool,    ITEM(res_dir.tls_authenticate), 0, 0, 0},
-   {"tlsenable",            store_bool,    ITEM(res_dir.tls_enable), 0, 0, 0},
-   {"tlsrequire",           store_bool,    ITEM(res_dir.tls_require), 0, 0, 0},
-   {"tlsverifypeer",        store_bool,    ITEM(res_dir.tls_verify_peer), 0, ITEM_DEFAULT, 1},
-   {"tlscacertificatefile", store_dir,       ITEM(res_dir.tls_ca_certfile), 0, 0, 0},
-   {"tlscacertificatedir",  store_dir,       ITEM(res_dir.tls_ca_certdir), 0, 0, 0},
-   {"tlscertificate",       store_dir,       ITEM(res_dir.tls_certfile), 0, 0, 0},
-   {"tlskey",               store_dir,       ITEM(res_dir.tls_keyfile), 0, 0, 0},
-   {"tlsdhfile",            store_dir,       ITEM(res_dir.tls_dhfile), 0, 0, 0},
-   {"tlsallowedcn",         store_alist_str, ITEM(res_dir.tls_allowed_cns), 0, 0, 0},
-   {"maximumbandwidthperjob", store_speed,     ITEM(res_dir.max_bandwidth_per_job), 0, 0, 0},
-   {"disablecommand",        store_alist_str, ITEM(res_dir.disable_cmds), 0, 0, 0},
+   {"Name",        store_name,     ITEM(res_dir.hdr.name),  0, ITEM_REQUIRED, 0},
+   {"Description", store_str,      ITEM(res_dir.hdr.desc),  0, 0, 0},
+   {"Password",    store_password, ITEM(res_dir.password),  0, ITEM_REQUIRED, 0},
+   {"Address",     store_str,      ITEM(res_dir.address),   0, 0, 0},
+   {"Monitor",     store_bool,   ITEM(res_dir.monitor),   0, ITEM_DEFAULT, 0},
+   {"TlsAuthenticate",      store_bool,    ITEM(res_dir.tls_authenticate), 0, 0, 0},
+   {"TlsEnable",            store_bool,    ITEM(res_dir.tls_enable), 0, 0, 0},
+   {"TlsRequire",           store_bool,    ITEM(res_dir.tls_require), 0, 0, 0},
+   {"TlsVerifyPeer",        store_bool,    ITEM(res_dir.tls_verify_peer), 0, ITEM_DEFAULT, 1},
+   {"TlsCaCertificateFile", store_dir,       ITEM(res_dir.tls_ca_certfile), 0, 0, 0},
+   {"TlsCaCertificateDir",  store_dir,       ITEM(res_dir.tls_ca_certdir), 0, 0, 0},
+   {"TlsCertificate",       store_dir,       ITEM(res_dir.tls_certfile), 0, 0, 0},
+   {"TlsKey",               store_dir,       ITEM(res_dir.tls_keyfile), 0, 0, 0},
+   {"TlsDhFile",            store_dir,       ITEM(res_dir.tls_dhfile), 0, 0, 0},
+   {"TlsAllowedCn",         store_alist_str, ITEM(res_dir.tls_allowed_cns), 0, 0, 0},
+   {"MaximumBandwidthPerJob", store_speed,     ITEM(res_dir.max_bandwidth_per_job), 0, 0, 0},
+   {"DisableCommand",        store_alist_str, ITEM(res_dir.disable_cmds), 0, 0, 0},
    {NULL, NULL, {0}, 0, 0, 0}
 };
 
@@ -146,10 +150,10 @@ extern RES_ITEM msgs_items[];
  * It must have one item for each of the resources.
  */
 RES_TABLE resources[] = {
-   {"director",      dir_items,   R_DIRECTOR},
-   {"filedaemon",    cli_items,   R_CLIENT},
-   {"client",        cli_items,   R_CLIENT},     /* alias for filedaemon */
-   {"messages",      msgs_items,  R_MSGS},
+   {"Director",      dir_items,   R_DIRECTOR},
+   {"FileDaemon",    cli_items,   R_CLIENT},
+   {"Messages",      msgs_items,  R_MSGS},
+   {"Client",        cli_items,   R_CLIENT},     /* alias for filedaemon */
    {NULL,            NULL,        0}
 };
 
@@ -224,9 +228,9 @@ static void store_digest_type(LEX *lc, RES_ITEM *item, int index, int pass)
 }
 
 /* Dump contents of resource */
-void dump_resource(int type, RES *reshdr, void sendit(void *sock, const char *fmt, ...), void *sock)
+void dump_resource(int type, RES *ares, void sendit(void *sock, const char *fmt, ...), void *sock)
 {
-   URES *res = (URES *)reshdr;
+   URES *res = (URES *)ares;
    int recurse = 1;
 
    if (res == NULL) {
@@ -239,11 +243,11 @@ void dump_resource(int type, RES *reshdr, void sendit(void *sock, const char *fm
    }
    switch (type) {
    case R_DIRECTOR:
-      sendit(sock, "Director: name=%s password=%s\n", reshdr->name,
+      sendit(sock, "Director: name=%s password=%s\n", ares->name,
               res->res_dir.password);
       break;
    case R_CLIENT:
-      sendit(sock, "Client: name=%s FDport=%d\n", reshdr->name,
+      sendit(sock, "Client: name=%s FDport=%d\n", ares->name,
               get_first_port_host_order(res->res_client.FDaddrs));
       break;
    case R_MSGS:
@@ -256,11 +260,13 @@ void dump_resource(int type, RES *reshdr, void sendit(void *sock, const char *fm
    default:
       sendit(sock, "Unknown resource type %d\n", type);
    }
+   ares = GetNextRes(type, ares);
    if (recurse && res->res_dir.hdr.next) {
       dump_resource(type, res->res_dir.hdr.next, sendit, sock);
    }
 }
 
+
 /*
  * Free memory of resource.
  * NB, we don't need to worry about freeing any references
@@ -343,7 +349,9 @@ void free_resource(RES *sres, int type)
       if (res->res_client.FDsrc_addr) {
          free_addresses(res->res_client.FDsrc_addr);
       }
-
+      if (res->res_client.snapshot_command) {
+         free(res->res_client.snapshot_command);
+      }
       if (res->res_client.pki_keypair_file) {
          free(res->res_client.pki_keypair_file);
       }
@@ -400,10 +408,12 @@ void free_resource(RES *sres, int type)
       }
       break;
    case R_MSGS:
-      if (res->res_msgs.mail_cmd)
+      if (res->res_msgs.mail_cmd) {
          free(res->res_msgs.mail_cmd);
-      if (res->res_msgs.operator_cmd)
+      }
+      if (res->res_msgs.operator_cmd) {
          free(res->res_msgs.operator_cmd);
+      }
       free_msgs_res((MSGS *)res);  /* free message resource */
       res = NULL;
       break;
@@ -436,7 +446,7 @@ void save_resource(int type, RES_ITEM *items, int pass)
    for (i=0; items[i].name; i++) {
       if (items[i].flags & ITEM_REQUIRED) {
             if (!bit_is_set(i, res_all.res_dir.hdr.item_present)) {
-               Emsg2(M_ABORT, 0, _("%s item is required in %s resource, but not found.\n"),
+               Emsg2(M_ERROR_TERM, 0, _("%s item is required in %s resource, but not found.\n"),
                  items[i].name, resources[rindex]);
              }
       }
@@ -459,6 +469,7 @@ void save_resource(int type, RES_ITEM *items, int pass)
                Emsg1(M_ABORT, 0, _("Cannot find Director resource %s\n"), res_all.res_dir.hdr.name);
             }
             res->res_dir.tls_allowed_cns = res_all.res_dir.tls_allowed_cns;
+            res->res_dir.disable_cmds = res_all.res_dir.disable_cmds;
             break;
          case R_CLIENT:
             if ((res = (URES *)GetResWithName(R_CLIENT, res_all.res_dir.hdr.name)) == NULL) {
@@ -471,6 +482,7 @@ void save_resource(int type, RES_ITEM *items, int pass)
             res->res_client.pki_recipients = res_all.res_client.pki_recipients;
 
             res->res_client.messages = res_all.res_client.messages;
+            res->res_client.disable_cmds = res_all.res_client.disable_cmds;
             break;
          default:
             Emsg1(M_ERROR, 0, _("Unknown resource type %d\n"), type);
