@@ -1,17 +1,20 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2003-2014 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2016 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from many
-   others, a complete list can be found in the file AUTHORS.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
    You may use this file and others of this release according to the
    license defined in the LICENSE file, which includes the Affero General
    Public License, v3.0 ("AGPLv3") and some additional permissions and
    terms pursuant to its AGPLv3 Section 7.
 
-   Bacula® is a registered trademark of Kern Sibbald.
+   This notice must be preserved when any source code is 
+   conveyed and/or propagated.
+
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
  *  This file handles the status command
@@ -24,32 +27,31 @@
 #include "bacula.h"
 #include "stored.h"
 #include "lib/status.h"
+#include "sd_plugins.h"
 
 /* Imported functions */
+extern void dbg_print_plugin(FILE *fp);
 
 /* Imported variables */
 extern BSOCK *filed_chan;
 extern void *start_heap;
 
 /* Static variables */
-static char qstatus[] = ".status %127s\n";
-
 static char OKqstatus[]   = "3000 OK .status\n";
 static char DotStatusJob[] = "JobId=%d JobStatus=%c JobErrors=%d\n";
 
 
 /* Forward referenced functions */
-static void sendit(const char *msg, int len, STATUS_PKT *sp);
 static void sendit(POOL_MEM &msg, int len, STATUS_PKT *sp);
 static void sendit(const char *msg, int len, void *arg);
-
+static void dbg_sendit(const char *msg, int len, void *arg);
 static void send_blocked_status(DEVICE *dev, STATUS_PKT *sp);
 static void send_device_status(DEVICE *dev, STATUS_PKT *sp);
-static void list_terminated_jobs(STATUS_PKT *sp);
 static void list_running_jobs(STATUS_PKT *sp);
 static void list_jobs_waiting_on_reservation(STATUS_PKT *sp);
 static void list_status_header(STATUS_PKT *sp);
-static void list_devices(STATUS_PKT *sp);
+static void list_devices(STATUS_PKT *sp, char *name=NULL);
+static void list_plugins(STATUS_PKT *sp);
 
 /*
  * Status command from Director
@@ -72,7 +74,7 @@ void output_status(STATUS_PKT *sp)
    list_jobs_waiting_on_reservation(sp);
 
    /*
-    * List terminated jobs
+    * List terminated jobs (defined in lib/status.h)
     */
    list_terminated_jobs(sp);
 
@@ -90,8 +92,12 @@ void output_status(STATUS_PKT *sp)
 
 
    list_spool_stats(sendit, (void *)sp);
+
    if (!sp->api) sendit("====\n\n", 6, sp);
 
+   if (chk_dbglvl(10)) {
+      dbg_print_plugin(stdout);
+   }
 }
 
 static void list_resources(STATUS_PKT *sp)
@@ -126,100 +132,150 @@ static find_device(char *devname)
 }
 #endif
 
-static void list_devices(STATUS_PKT *sp)
+
+static void list_one_device(char *name, DEVICE *dev, STATUS_PKT *sp)
 {
-   DEVRES *device;
-   AUTOCHANGER *changer;
-   DEVICE *dev;
    char b1[35], b2[35], b3[35];
    POOL_MEM msg(PM_MESSAGE);
    int len;
    int bpb;
 
-   len = Mmsg(msg, _("\nDevice status:\n"));
-   if (!sp->api) sendit(msg, len, sp);
+   if (!dev) {
+      len = Mmsg(msg, _("\nDevice \"%s\" is not open or does not exist.\n"),
+                 name);
+      sendit(msg, len, sp);
+      if (!sp->api) sendit("==\n", 4, sp);
+      return;
+   }
 
-   foreach_res(changer, R_AUTOCHANGER) {
-      len = Mmsg(msg, _("Autochanger \"%s\" with devices:\n"),
-         changer->hdr.name);
+   if (dev->is_open()) {
+      if (dev->is_labeled()) {
+         len = Mmsg(msg, _("\nDevice %s is %s %s:\n"
+                           "    Volume:      %s\n"
+                           "    Pool:        %s\n"
+                           "    Media type:  %s\n"),
+            dev->print_type(), dev->print_name(),
+            dev->blocked()?_("waiting for"):_("mounted with"),
+            dev->VolHdr.VolumeName,
+            dev->pool_name[0]?dev->pool_name:_("*unknown*"),
+            dev->device->media_type);
+         sendit(msg, len, sp);
+      } else {
+         len = Mmsg(msg, _("\nDevice %s: %s open but no Bacula volume is currently mounted.\n"),
+            dev->print_type(), dev->print_name());
+         sendit(msg, len, sp);
+      }
+      send_blocked_status(dev, sp);
+      if (dev->can_append()) {
+         bpb = dev->VolCatInfo.VolCatBlocks;
+         if (bpb <= 0) {
+            bpb = 1;
+         }
+         bpb = dev->VolCatInfo.VolCatBytes / bpb;
+         len = Mmsg(msg, _("    Total Bytes=%s Blocks=%s Bytes/block=%s\n"),
+            edit_uint64_with_commas(dev->VolCatInfo.VolCatBytes, b1),
+            edit_uint64_with_commas(dev->VolCatInfo.VolCatBlocks, b2),
+            edit_uint64_with_commas(bpb, b3));
+         sendit(msg, len, sp);
+      } else {  /* reading */
+         bpb = dev->VolCatInfo.VolCatReads;
+         if (bpb <= 0) {
+            bpb = 1;
+         }
+         if (dev->VolCatInfo.VolCatRBytes > 0) {
+            bpb = dev->VolCatInfo.VolCatRBytes / bpb;
+         } else {
+            bpb = 0;
+         }
+         len = Mmsg(msg, _("    Total Bytes Read=%s Blocks Read=%s Bytes/block=%s\n"),
+            edit_uint64_with_commas(dev->VolCatInfo.VolCatRBytes, b1),
+            edit_uint64_with_commas(dev->VolCatInfo.VolCatReads, b2),
+            edit_uint64_with_commas(bpb, b3));
+         sendit(msg, len, sp);
+      }
+      len = Mmsg(msg, _("    Positioned at File=%s Block=%s\n"),
+         edit_uint64_with_commas(dev->file, b1),
+         edit_uint64_with_commas(dev->block_num, b2));
       sendit(msg, len, sp);
 
-      foreach_alist(device, changer->device) {
-         if (device->dev) {
-            len = Mmsg(msg, "   %s\n", device->dev->print_name());
-            sendit(msg, len, sp);
-         } else {
-            len = Mmsg(msg, "   %s\n", device->hdr.name);
-            sendit(msg, len, sp);
-         }
+   } else {
+      len = Mmsg(msg, _("\nDevice %s: %s is not open.\n"),
+                 dev->print_type(), dev->print_name());
+      sendit(msg, len, sp);
+      send_blocked_status(dev, sp);
+   }
+
+   /* TODO: We need to check with Mount command, maybe we can
+    * display this number only when the device is open.
+    */
+   if (dev->is_file()) {
+      char ed1[50];
+      uint64_t f, t;
+      dev->get_freespace(&f, &t);
+      if (t > 0) {              /* We might not have access to numbers */
+         len = Mmsg(msg, _("    Available Space=%sB\n"),
+                    edit_uint64_with_suffix(f, ed1));
+         sendit(msg, len, sp);
       }
    }
 
+   if (!sp->api) sendit("==\n", 4, sp);
+}
+
+void _dbg_list_one_device(char *name, DEVICE *dev, const char *file, int line)
+{
+   STATUS_PKT sp;
+   sp.bs = NULL;
+   sp.callback = dbg_sendit;
+   sp.context = NULL;
+   d_msg(file, line, 0, "Called dbg_list_one_device():");
+   list_one_device(name, dev, &sp);
+   send_device_status(dev, &sp);
+}
+
+static void list_one_autochanger(char *name, AUTOCHANGER *changer, STATUS_PKT *sp)
+{
+   int     len;
+   DEVRES *device;
+   POOL_MEM msg(PM_MESSAGE);
+
+   len = Mmsg(msg, _("Autochanger \"%s\" with devices:\n"),
+              changer->hdr.name);
+   sendit(msg, len, sp);
+
+   foreach_alist(device, changer->device) {
+      if (device->dev) {
+         len = Mmsg(msg, "   %s\n", device->dev->print_name());
+         sendit(msg, len, sp);
+      } else {
+         len = Mmsg(msg, "   %s\n", device->hdr.name);
+         sendit(msg, len, sp);
+      }
+   }
+}
+
+static void list_devices(STATUS_PKT *sp, char *name)
+{
+   int len;
+   DEVRES *device;
+   AUTOCHANGER *changer;
+   POOL_MEM msg(PM_MESSAGE);
+
+   if (!sp->api) {
+      len = Mmsg(msg, _("\nDevice status:\n"));
+      sendit(msg, len, sp);
+   }
+
+   foreach_res(changer, R_AUTOCHANGER) {
+      if (!name || strcmp(changer->hdr.name, name) == 0) {
+         list_one_autochanger(changer->hdr.name, changer, sp);
+      }
+   }
 
    foreach_res(device, R_DEVICE) {
-      dev = device->dev;
-      if (dev && dev->is_open()) {
-         if (dev->is_labeled()) {
-            len = Mmsg(msg, _("\nDevice %s is %s:\n"
-                              "    Volume:      %s\n"
-                              "    Pool:        %s\n"
-                              "    Media type:  %s\n"),
-               dev->print_name(),
-               dev->blocked()?_("waiting for"):_("mounted with"),
-               dev->VolHdr.VolumeName,
-               dev->pool_name[0]?dev->pool_name:_("*unknown*"),
-               dev->device->media_type);
-            sendit(msg, len, sp);
-         } else {
-            len = Mmsg(msg, _("\nDevice %s open but no Bacula volume is currently mounted.\n"),
-               dev->print_name());
-            sendit(msg, len, sp);
-         }
-         send_blocked_status(dev, sp);
-         if (dev->can_append()) {
-            bpb = dev->VolCatInfo.VolCatBlocks;
-            if (bpb <= 0) {
-               bpb = 1;
-            }
-            bpb = dev->VolCatInfo.VolCatBytes / bpb;
-            len = Mmsg(msg, _("    Total Bytes=%s Blocks=%s Bytes/block=%s\n"),
-               edit_uint64_with_commas(dev->VolCatInfo.VolCatBytes, b1),
-               edit_uint64_with_commas(dev->VolCatInfo.VolCatBlocks, b2),
-               edit_uint64_with_commas(bpb, b3));
-            sendit(msg, len, sp);
-         } else {  /* reading */
-            bpb = dev->VolCatInfo.VolCatReads;
-            if (bpb <= 0) {
-               bpb = 1;
-            }
-            if (dev->VolCatInfo.VolCatRBytes > 0) {
-               bpb = dev->VolCatInfo.VolCatRBytes / bpb;
-            } else {
-               bpb = 0;
-            }
-            len = Mmsg(msg, _("    Total Bytes Read=%s Blocks Read=%s Bytes/block=%s\n"),
-               edit_uint64_with_commas(dev->VolCatInfo.VolCatRBytes, b1),
-               edit_uint64_with_commas(dev->VolCatInfo.VolCatReads, b2),
-               edit_uint64_with_commas(bpb, b3));
-            sendit(msg, len, sp);
-         }
-         len = Mmsg(msg, _("    Positioned at File=%s Block=%s\n"),
-            edit_uint64_with_commas(dev->file, b1),
-            edit_uint64_with_commas(dev->block_num, b2));
-         sendit(msg, len, sp);
-
-      } else {
-         if (dev) {
-            len = Mmsg(msg, _("\nDevice %s is not open.\n"), dev->print_name());
-            sendit(msg, len, sp);
-            send_blocked_status(dev, sp);
-        } else {
-            len = Mmsg(msg, _("\nDevice \"%s\" is not open or does not exist.\n"), device->hdr.name);
-            sendit(msg, len, sp);
-         }
+      if (!name || strcmp(device->hdr.name, name) == 0) {
+         list_one_device(device->hdr.name, device->dev, sp);
       }
-
-      if (!sp->api) sendit("==\n", 4, sp);
    }
    if (!sp->api) sendit("====\n\n", 6, sp);
 }
@@ -231,8 +287,8 @@ static void list_status_header(STATUS_PKT *sp)
    POOL_MEM msg(PM_MESSAGE);
    int len;
 
-   len = Mmsg(msg, _("%s Version: %s (%s) %s %s %s\n"),
-              my_name, VERSION, BDATE, HOST_OS, DISTNAME, DISTVER);
+   len = Mmsg(msg, _("%s %sVersion: %s (%s) %s %s %s\n"),
+              my_name, "", VERSION, BDATE, HOST_OS, DISTNAME, DISTVER);
    sendit(msg, len, sp);
 
    bstrftime_nc(dt, sizeof(dt), daemon_start_time);
@@ -251,23 +307,9 @@ static void list_status_header(STATUS_PKT *sp)
    len = Mmsg(msg, " Sizes: boffset_t=%d size_t=%d int32_t=%d int64_t=%d "
               "mode=%d,%d\n",
               (int)sizeof(boffset_t), (int)sizeof(size_t), (int)sizeof(int32_t),
-              (int)sizeof(int64_t), (int)DEVELOPER_MODE, (int)BEEF);
+              (int)sizeof(int64_t), (int)DEVELOPER_MODE, (int)0);
    sendit(msg, len, sp);
-   if (bplugin_list->size() > 0) {
-      Plugin *plugin;
-      int len;
-      pm_strcpy(msg, " Plugin: ");
-      foreach_alist(plugin, bplugin_list) {
-         len = pm_strcat(msg, plugin->file);
-         if (len > 80) {
-            pm_strcat(msg, "\n   ");
-         } else {
-            pm_strcat(msg, " ");
-         }
-      }
-      len = pm_strcat(msg, "\n");
-      sendit(msg.c_str(), len, sp);
-   }
+   list_plugins(sp);
 }
 
 static void send_blocked_status(DEVICE *dev, STATUS_PKT *sp)
@@ -280,6 +322,10 @@ static void send_blocked_status(DEVICE *dev, STATUS_PKT *sp)
       sendit(msg, len, sp);
       return;
    }
+   if (!dev->enabled) {
+      len = Mmsg(msg, _("    Device is disabled. User command.\n"));
+      sendit(msg, len, sp);
+   }
    switch (dev->blocked()) {
    case BST_UNMOUNTED:
       len = Mmsg(msg, _("    Device is BLOCKED. User unmounted.\n"));
@@ -350,7 +396,7 @@ static void send_blocked_status(DEVICE *dev, STATUS_PKT *sp)
    }
 }
 
-static void send_device_status(DEVICE *dev, STATUS_PKT *sp)
+void send_device_status(DEVICE *dev, STATUS_PKT *sp)
 {
    POOL_MEM msg(PM_MESSAGE);
    int len;
@@ -392,11 +438,11 @@ static void send_device_status(DEVICE *dev, STATUS_PKT *sp)
       dev->state & ST_SHORT ? "" : "!",
       dev->state & ST_MOUNTED ? "" : "!");
    sendit(msg, len, sp);
-   len = Mmsg(msg, _("  num_writers=%d reserves=%d block=%d\n"), dev->num_writers,
-              dev->num_reserved(), dev->blocked());
+   len = Mmsg(msg, _("  num_writers=%d reserves=%d block=%d enabled=%d\n"), dev->num_writers,
+              dev->num_reserved(), dev->blocked(), dev->enabled);
    sendit(msg, len, sp);
 
-   len = Mmsg(msg, _("Attached JobsIds: "));
+   len = Mmsg(msg, _("Attached JobIds: "));
    sendit(msg, len, sp);
    dev->Lock();
    dev->Lock_dcrs();
@@ -428,7 +474,8 @@ static void send_device_status(DEVICE *dev, STATUS_PKT *sp)
 static void list_running_jobs(STATUS_PKT *sp)
 {
    bool found = false;
-   int avebps, bps, sec;
+   uint64_t inst_bps, total_bps;
+   int inst_sec, total_sec;
    JCR *jcr;
    DCR *dcr, *rdcr;
    char JobName[MAX_NAME_LENGTH];
@@ -469,8 +516,7 @@ static void list_running_jobs(STATUS_PKT *sp)
                    rdcr->dev?rdcr->dev->print_name():
                             rdcr->device->device_name);
             sendit(msg, len, sp);
-         }
-         if (dcr && dcr->device) {
+         } else if (dcr && dcr->device) {
             len = Mmsg(msg, _("Writing: %s %s job %s JobId=%d Volume=\"%s\"\n"
                             "    pool=\"%s\" device=%s\n"),
                    job_level_to_str(jcr->getJobLevel()),
@@ -489,24 +535,35 @@ static void list_running_jobs(STATUS_PKT *sp)
          if (jcr->last_time == 0) {
             jcr->last_time = jcr->run_time;
          }
-         sec = now - jcr->last_time;
-         if (sec <= 0) {
-            sec = 1;
+         total_sec = now - jcr->run_time;
+         inst_sec = now - jcr->last_time;
+         if (total_sec <= 0) {
+            total_sec = 1;
          }
-         bps = (jcr->JobBytes - jcr->LastJobBytes) / sec;
+         if (inst_sec <= 0) {
+            inst_sec = 1;
+         }
+         /* Instanteous bps not smoothed */
+         inst_bps = (jcr->JobBytes - jcr->LastJobBytes) / inst_sec;
          if (jcr->LastRate == 0) {
-            jcr->LastRate = bps;
+            jcr->LastRate = inst_bps;
          }
-         avebps = (jcr->LastRate + bps) / 2;
+         /* Smooth the instantaneous bps a bit */
+         inst_bps = (2 * jcr->LastRate + inst_bps) / 3;
+         /* total bps (AveBytes/sec) since start of job */
+         total_bps = jcr->JobBytes / total_sec;
          len = Mmsg(msg, _("    Files=%s Bytes=%s AveBytes/sec=%s LastBytes/sec=%s\n"),
             edit_uint64_with_commas(jcr->JobFiles, b1),
             edit_uint64_with_commas(jcr->JobBytes, b2),
-            edit_uint64_with_commas(avebps, b3),
-            edit_uint64_with_commas(bps, b4));
+            edit_uint64_with_commas(total_bps, b3),
+            edit_uint64_with_commas(inst_bps, b4));
          sendit(msg, len, sp);
-         jcr->LastRate = avebps;
-         jcr->LastJobBytes = jcr->JobBytes;
-         jcr->last_time = now;
+         /* Update only every 10 seconds */
+         if (now - jcr->last_time > 10) {
+            jcr->LastRate = inst_bps;
+            jcr->LastJobBytes = jcr->JobBytes;
+            jcr->last_time = now;
+         }
          found = true;
 #ifdef DEBUG
          if (jcr->file_bsock) {
@@ -551,9 +608,7 @@ static void list_jobs_waiting_on_reservation(STATUS_PKT *sp)
    if (!sp->api) sendit("====\n", 5, sp);
 }
 
-/*
- * Send to Director
- */
+
 static void sendit(const char *msg, int len, void *sp)
 {
    sendit(msg, len, (STATUS_PKT *)sp);
@@ -563,6 +618,7 @@ static void sendit(POOL_MEM &msg, int len, STATUS_PKT *sp)
 {
    BSOCK *bs = sp->bs;
    if (bs) {
+      bs->msg = check_pool_memory_size(bs->msg, len+1);
       memcpy(bs->msg, msg.c_str(), len+1);
       bs->msglen = len+1;
       bs->send();
@@ -571,6 +627,12 @@ static void sendit(POOL_MEM &msg, int len, STATUS_PKT *sp)
    }
 }
 
+static void dbg_sendit(const char *msg, int len, void *sp)
+{
+   if (len > 0) {
+      Dmsg0(-1, msg);
+   }
+}
 
 /*
  * Status command from Director
@@ -593,122 +655,123 @@ bool status_cmd(JCR *jcr)
 bool qstatus_cmd(JCR *jcr)
 {
    BSOCK *dir = jcr->dir_bsock;
-   POOL_MEM cmd;
    JCR *njcr;
    s_last_job* job;
    STATUS_PKT sp;
+   POOLMEM *args = get_pool_memory(PM_MESSAGE);
+   char *argk[MAX_CMD_ARGS];          /* argument keywords */
+   char *argv[MAX_CMD_ARGS];          /* argument values */
+   int argc;                          /* number of arguments */
+   bool ret=true;
+   char *cmd;
+   char *device=NULL;
+   int api = true;
 
    sp.bs = dir;
-   if (sscanf(dir->msg, qstatus, cmd.c_str()) != 1) {
+
+   parse_args(dir->msg, &args, &argc, argk, argv, MAX_CMD_ARGS);
+
+   /* .status xxxx at the minimum */
+   if (argc < 2 || strcmp(argk[0], ".status") != 0) {
       pm_strcpy(jcr->errmsg, dir->msg);
       dir->fsend(_("3900 No arg in .status command: %s\n"), jcr->errmsg);
       dir->signal(BNET_EOD);
       return false;
    }
+
+   cmd = argk[1];
    unbash_spaces(cmd);
 
-   Dmsg1(200, "cmd=%s\n", cmd.c_str());
+   /* The status command can contain some arguments
+    * i=0 => .status
+    * i=1 => [running | current | last | ... ]
+    */
+   for (int i=0 ; i < argc ; i++) {
+      if (!strcmp(argk[i], "device") && argv[i]) {
+         device = argv[i];
+         unbash_spaces(device);
 
-   if (strcasecmp(cmd.c_str(), "current") == 0) {
-      dir->fsend(OKqstatus, cmd.c_str());
+      } else if (!strcmp(argk[i], "api") && argv[i]) {
+         api = atoi(argv[i]);
+
+      } else if (!strcmp(argk[i], "api_opts") && argv[i]) {
+         strncpy(sp.api_opts, argv[i], sizeof(sp.api_opts));;
+      }
+   }
+
+   Dmsg1(100, "cmd=%s\n", cmd);
+
+   if (strcasecmp(cmd, "current") == 0) {
+      dir->fsend(OKqstatus, cmd);
       foreach_jcr(njcr) {
          if (njcr->JobId != 0) {
             dir->fsend(DotStatusJob, njcr->JobId, njcr->JobStatus, njcr->JobErrors);
          }
       }
       endeach_jcr(njcr);
-   } else if (strcasecmp(cmd.c_str(), "last") == 0) {
-      dir->fsend(OKqstatus, cmd.c_str());
+   } else if (strcasecmp(cmd, "last") == 0) {
+      dir->fsend(OKqstatus, cmd);
       if ((last_jobs) && (last_jobs->size() > 0)) {
          job = (s_last_job*)last_jobs->last();
          dir->fsend(DotStatusJob, job->JobId, job->JobStatus, job->Errors);
       }
-   } else if (strcasecmp(cmd.c_str(), "header") == 0) {
-       sp.api = true;
+   } else if (strcasecmp(cmd, "header") == 0) {
+       sp.api = api;
        list_status_header(&sp);
-   } else if (strcasecmp(cmd.c_str(), "running") == 0) {
-       sp.api = true;
+   } else if (strcasecmp(cmd, "running") == 0) {
+       sp.api = api;
        list_running_jobs(&sp);
-   } else if (strcasecmp(cmd.c_str(), "waitreservation") == 0) {
-       sp.api = true;
+   } else if (strcasecmp(cmd, "waitreservation") == 0) {
+       sp.api = api;
        list_jobs_waiting_on_reservation(&sp);
-   } else if (strcasecmp(cmd.c_str(), "devices") == 0) {
-       sp.api = true;
-       list_devices(&sp);
-   } else if (strcasecmp(cmd.c_str(), "volumes") == 0) {
-       sp.api = true;
+   } else if (strcasecmp(cmd, "devices") == 0) {
+       sp.api = api;
+       list_devices(&sp, device);
+   } else if (strcasecmp(cmd, "volumes") == 0) {
+       sp.api = api;
        list_volumes(sendit, &sp);
-   } else if (strcasecmp(cmd.c_str(), "spooling") == 0) {
-       sp.api = true;
+   } else if (strcasecmp(cmd, "spooling") == 0) {
+       sp.api = api;
        list_spool_stats(sendit, &sp);
-   } else if (strcasecmp(cmd.c_str(), "terminated") == 0) {
-       sp.api = true;
-       list_terminated_jobs(&sp);
-   } else if (strcasecmp(cmd.c_str(), "resources") == 0) {
-       sp.api = true;
+   } else if (strcasecmp(cmd, "terminated") == 0) {
+       sp.api = api;
+       list_terminated_jobs(&sp); /* defined in lib/status.h */
+   } else if (strcasecmp(cmd, "resources") == 0) {
+       sp.api = api;
        list_resources(&sp);
    } else {
       pm_strcpy(jcr->errmsg, dir->msg);
       dir->fsend(_("3900 Unknown arg in .status command: %s\n"), jcr->errmsg);
       dir->signal(BNET_EOD);
-      return false;
+      ret = false;
    }
    dir->signal(BNET_EOD);
-   return true;
+   free_pool_memory(args);
+   return ret;
 }
 
-#if defined(HAVE_WIN32)
-int bacstat = 0;
-
-/* Return a one line status for the tray monitor */
-char *bac_status(char *buf, int buf_len)
+static void list_plugins(STATUS_PKT *sp)
 {
-   JCR *njcr;
-   const char *termstat = _("Bacula Storage: Idle");
-   struct s_last_job *job;
-   int stat = 0;                      /* Idle */
-
-   if (!last_jobs) {
-      goto done;
-   }
-   Dmsg0(1000, "Begin bac_status jcr loop.\n");
-   foreach_jcr(njcr) {
-      if (njcr->JobId != 0) {
-         stat = JS_Running;
-         termstat = _("Bacula Storage: Running");
-         break;
-      }
-   }
-   endeach_jcr(njcr);
-
-   if (stat != 0) {
-      goto done;
-   }
-   if (last_jobs->size() > 0) {
-      job = (struct s_last_job *)last_jobs->last();
-      stat = job->JobStatus;
-      switch (job->JobStatus) {
-      case JS_Canceled:
-         termstat = _("Bacula Storage: Last Job Canceled");
-         break;
-      case JS_ErrorTerminated:
-      case JS_FatalError:
-         termstat = _("Bacula Storage: Last Job Failed");
-         break;
-      default:
-         if (job->Errors) {
-            termstat = _("Bacula Storage: Last Job had Warnings");
+   POOL_MEM msg(PM_MESSAGE);
+   if (b_plugin_list && b_plugin_list->size() > 0) {
+      Plugin *plugin;
+      int len;
+      pm_strcpy(msg, " Plugin: ");
+      foreach_alist(plugin, b_plugin_list) {
+         len = pm_strcat(msg, plugin->file);
+         /* Print plugin version when debug activated */
+         if (debug_level > 0 && plugin->pinfo) {
+            pm_strcat(msg, "(");
+            pm_strcat(msg, NPRT(sdplug_info(plugin)->plugin_version));
+            len = pm_strcat(msg, ")");
+         }
+         if (len > 80) {
+            pm_strcat(msg, "\n   ");
+         } else {
+            pm_strcat(msg, " ");
          }
-         break;
       }
+      len = pm_strcat(msg, "\n");
+      sendit(msg.c_str(), len, sp);
    }
-   Dmsg0(1000, "End bac_status jcr loop.\n");
-done:
-   bacstat = stat;
-   if (buf) {
-      bstrncpy(buf, termstat, buf_len);
-   }
-   return buf;
 }
-
-#endif /* HAVE_WIN32 */
